<?php

namespace Drupal\layout_builder_context\EventSubscriber;

use Drupal\context\ContextManager;
use Drupal\layout_builder\Event\SectionComponentBuildRenderArrayEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\layout_builder\LayoutBuilderEvents;
use Drupal\context\Entity\Context;
use Drupal\Core\Cache\Cache;

/**
 * Event subscriber to the initial render array.
 */
class BlockComponentRenderArraySubscriber implements EventSubscriberInterface {

  /**
   * The context manager service.
   *
   * @var \Drupal\context\ContextManager
   */
  protected $contextManager;

  /**
   * BlockComponentRenderArraySubscriber constructor.
   *
   * @param \Drupal\context\ContextManager $context_manager
   *   The context manager service.
   */
  public function __construct(ContextManager $context_manager) {
    $this->contextManager = $context_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[LayoutBuilderEvents::SECTION_COMPONENT_BUILD_RENDER_ARRAY] = [
      'onBuildRender',
      50,
    ];
    return $events;
  }

  /**
   * Add each component Context settings to the block.
   *
   * @param \Drupal\layout_builder\Event\SectionComponentBuildRenderArrayEvent $event
   *   The section component render event.
   */
  public function onBuildRender(SectionComponentBuildRenderArrayEvent $event) {
    $build = $event->getBuild();

    if (empty($build) || $event->inPreview()) {
      return;
    }

    $context_id = $event->getComponent()->get('context_visibility');

    if ($context_id) {
      $build['#context_visibility'] = $context_id;

      /** @var \Drupal\context\Entity\Context $context */
      $context = $this->contextManager->getContext($context_id);

      if ($context instanceof Context) {
        $visible = $this->contextManager->evaluateContextConditions($context);
        $cache_contexts = $context->getCacheContexts();
        $cache_tags = $context->getCacheTags();

        if (!empty($cache_contexts)) {
          $block_cache_contexts = $build['#cache']['contexts'] ?? [];
          $build['#cache']['contexts'] = Cache::mergeContexts($block_cache_contexts, $cache_contexts);
        }

        if (!empty($cache_tags)) {
          $block_cache_tags = $build['#cache']['tags'] ?? [];
          $build['#cache']['tags'] = Cache::mergeTags($block_cache_tags, $cache_tags);
        }

        if (!$visible) {
          // Should the event stop propogating here or do we want the cache metadata to bubble?
          unset($build['content']);
        }
      }
    }

    $event->setBuild($build);
  }

}
